"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CdkPipeline = exports.DotnetVersion = exports.JavaVersion = exports.GoVersion = exports.NodeVersion = void 0;
const constructs_1 = require("constructs");
const aws_codepipeline_1 = require("aws-cdk-lib/aws-codepipeline");
const aws_kms_1 = require("aws-cdk-lib/aws-kms");
const artifact_bucket_1 = require("./artifact-bucket");
const pipelines_1 = require("aws-cdk-lib/pipelines");
const aws_codebuild_1 = require("aws-cdk-lib/aws-codebuild");
const pipeline_notification_rule_1 = require("./pipeline-notification-rule");
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_codecommit_1 = require("aws-cdk-lib/aws-codecommit");
const enums_1 = require("./enums");
const aws_iam_1 = require("aws-cdk-lib/aws-iam");
/**
 * Node runtimes supported by CodebBuild
 * See https://docs.aws.amazon.com/codebuild/latest/userguide/available-runtimes.html
 */
var NodeVersion;
(function (NodeVersion) {
    NodeVersion["NODE_16"] = "16";
    NodeVersion["NODE_18"] = "18";
    NodeVersion["NODE_20"] = "20";
})(NodeVersion || (exports.NodeVersion = NodeVersion = {}));
/**
 * Go runtimes supported by CodeBuild
 * See https://docs.aws.amazon.com/codebuild/latest/userguide/available-runtimes.html
 */
var GoVersion;
(function (GoVersion) {
    GoVersion["GO_1_12"] = "1.12";
    GoVersion["GO_1_13"] = "1.13";
    GoVersion["GO_1_14"] = "1.14";
    GoVersion["GO_1_15"] = "1.15";
    GoVersion["GO_1_16"] = "1.16";
    GoVersion["GO_1_17"] = "1.17";
    GoVersion["GO_1_18"] = "1.18";
    GoVersion["GO_1_20"] = "1.20";
    GoVersion["GO_1_21"] = "1.21";
})(GoVersion || (exports.GoVersion = GoVersion = {}));
/**
 * Java runtimes supported by CodeBuild
 * See https://docs.aws.amazon.com/codebuild/latest/userguide/available-runtimes.html
 */
var JavaVersion;
(function (JavaVersion) {
    JavaVersion["JAVA_8"] = "corretto8";
    JavaVersion["JAVA_11"] = "corretto11";
    JavaVersion["JAVA_16"] = "corretto17";
    JavaVersion["JAVA_21"] = "corretto21";
})(JavaVersion || (exports.JavaVersion = JavaVersion = {}));
/**
 * Dotnet runtimes supported by CodeBuild
 * See https://docs.aws.amazon.com/codebuild/latest/userguide/available-runtimes.htmldotnet
 */
var DotnetVersion;
(function (DotnetVersion) {
    DotnetVersion["DOTNET_3_1"] = "3.1";
    DotnetVersion["DOTNET_5_0"] = "5.0";
    DotnetVersion["DOTNET_6_0"] = "6.0";
})(DotnetVersion || (exports.DotnetVersion = DotnetVersion = {}));
/**
 * An abstraction to ease CDK pipeline creation and configuration.
 */
class CdkPipeline extends constructs_1.Construct {
    constructor(scope, id, props) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        super(scope, id);
        const encryptionKey = aws_kms_1.Key.fromKeyArn(this, 'EncryptionKey', props.keyArn);
        const artifactBucket = new artifact_bucket_1.ArtifactBucket(this, 'ArtifactBucket', {
            encryptionKey,
            accountIds: props.accountIds,
        });
        const underlyingPipeline = new aws_codepipeline_1.Pipeline(this, 'Pipeline', {
            artifactBucket,
            pipelineName: props.pipelineName,
        });
        let input;
        if (props.connectionArn !== undefined) {
            input = pipelines_1.CodePipelineSource.connection(props.repository, props.branch, {
                connectionArn: props.connectionArn,
            });
        }
        else {
            const repository = props.repository.startsWith('arn:')
                ? aws_codecommit_1.Repository.fromRepositoryArn(this, 'Repository', props.repository)
                : aws_codecommit_1.Repository.fromRepositoryName(this, 'Repository', props.repository);
            input = pipelines_1.CodePipelineSource.codeCommit(repository, props.branch);
        }
        const stackName = aws_cdk_lib_1.Stack.of(this).stackName;
        const cdkDirectory = (_a = props.cdkDirectory) !== null && _a !== void 0 ? _a : '.';
        let commands = props.commands;
        if (commands === undefined &&
            props.packageManager === enums_1.NodePackageManager.PNPM) {
            commands = [
                `cd ${cdkDirectory !== null && cdkDirectory !== void 0 ? cdkDirectory : '.'}`,
                'npm -g i pnpm',
                'pnpm i --frozen-lockfile --prefer-offline',
                'pnpm run build',
                'pnpm run test',
                `pnpx cdk synth ${stackName}`,
            ];
        }
        else if (commands === undefined) {
            commands = [
                `cd ${cdkDirectory !== null && cdkDirectory !== void 0 ? cdkDirectory : '.'}`,
                'npm ci --prefer-offline',
                'npm run build',
                'npm run test',
                `npx cdk synth ${stackName}`,
            ];
        }
        const stack = aws_cdk_lib_1.Stack.of(this);
        const codeArtifactDomains = (_b = props.codeArtifactDomains) === null || _b === void 0 ? void 0 : _b.map(domain => {
            return domain.startsWith('arn:aws')
                ? domain
                : aws_cdk_lib_1.Arn.format({
                    service: 'codeartifact',
                    resource: 'domain',
                    resourceName: domain,
                }, stack);
        });
        const codeArtifactRepositories = (_c = props.codeArtifactRepositories) === null || _c === void 0 ? void 0 : _c.map(repository => {
            return repository.startsWith('arn:aws')
                ? repository
                : aws_cdk_lib_1.Arn.format({
                    service: 'codeartifact',
                    resource: 'repository',
                    resourceName: repository,
                }, stack);
        });
        const rolePolicy = (_d = props.rolePolicy) !== null && _d !== void 0 ? _d : [];
        if (codeArtifactDomains) {
            rolePolicy.push(new aws_iam_1.PolicyStatement({
                resources: codeArtifactDomains,
                actions: ['codeartifact:GetAuthorizationToken'],
                effect: aws_iam_1.Effect.ALLOW,
            }));
            rolePolicy.push(new aws_iam_1.PolicyStatement({
                resources: ['*'],
                actions: ['sts:GetServiceBearerToken'],
                effect: aws_iam_1.Effect.ALLOW,
            }));
        }
        if (codeArtifactRepositories) {
            rolePolicy.push(new aws_iam_1.PolicyStatement({
                resources: codeArtifactRepositories,
                actions: [
                    'codeartifact:DescribePackageVersion',
                    'codeartifact:DescribeRepository',
                    'codeartifact:GetPackageVersionReadme',
                    'codeartifact:GetRepositoryEndpoint',
                    'codeartifact:ListPackages',
                    'codeartifact:ListPackageVersions',
                    'codeartifact:ListPackageVersionAssets',
                    'codeartifact:ListPackageVersionDependencies',
                    'codeartifact:ReadFromRepository',
                ],
                effect: aws_iam_1.Effect.ALLOW,
            }));
        }
        this.pipeline = new pipelines_1.CodePipeline(this, 'CodePipeline', {
            codePipeline: underlyingPipeline,
            selfMutation: (_e = props.selfMutation) !== null && _e !== void 0 ? _e : true,
            dockerEnabledForSynth: (_f = props.dockerEnabledForSynth) !== null && _f !== void 0 ? _f : true,
            dockerEnabledForSelfMutation: (_g = props.dockerEnabledForSelfMutation) !== null && _g !== void 0 ? _g : true,
            publishAssetsInParallel: (_h = props.publishAssetsInParallel) !== null && _h !== void 0 ? _h : false,
            synth: new pipelines_1.ShellStep('Synth', {
                primaryOutputDirectory: `${cdkDirectory}/cdk.out`,
                input,
                commands,
                additionalInputs: (_j = props.additionalInputs) !== null && _j !== void 0 ? _j : {},
            }),
            synthCodeBuildDefaults: {
                partialBuildSpec: aws_codebuild_1.BuildSpec.fromObject({
                    cache: {
                        paths: ['/root/.npm/**/*', '/root/.pnpm-store/**/*'],
                    },
                    phases: {
                        install: {
                            'runtime-versions': {
                                nodejs: (_k = props.nodeVersion) !== null && _k !== void 0 ? _k : NodeVersion.NODE_18,
                                go: props.goVersion,
                                java: props.javaVersion,
                                dotnet: props.dotnetVersion,
                            },
                            commands: ['npm config set fund false', 'npm -g i esbuild']
                                .concat((_l = props.additionalInstallCommands) !== null && _l !== void 0 ? _l : [])
                                .concat(['node --version']),
                        },
                    },
                }),
                buildEnvironment: {
                    privileged: true,
                    computeType: (_m = props.computeType) !== null && _m !== void 0 ? _m : aws_codebuild_1.ComputeType.SMALL,
                    buildImage: (_o = props.buildImage) !== null && _o !== void 0 ? _o : aws_codebuild_1.LinuxBuildImage.AMAZON_LINUX_2_5,
                },
                rolePolicy,
            },
        });
        // Handle pipeline notifications
        if (props.slackChannelConfiguration && props.slackChannelConfigurationArn) {
            throw new Error('Only one of slackChannelConfiguration and slackChannelConfigurationArn can be specified');
        }
        if (props.notificationTopic && props.notificationTopicArn) {
            throw new Error('Only one of notificationTopic and notificationTopicArn can be specified');
        }
        if (props.slackChannelConfiguration ||
            props.slackChannelConfigurationArn ||
            props.notificationTopicArn ||
            props.notificationTopic) {
            this.pipelineNotificationRule = new pipeline_notification_rule_1.PipelineNotificationRule(this, 'Notification', {
                events: props.notificationEvents,
                source: underlyingPipeline,
            });
            if (props.slackChannelConfiguration) {
                this.pipelineNotificationRule.addSlackChannel(props.slackChannelConfiguration);
            }
            else if (props.slackChannelConfigurationArn) {
                this.pipelineNotificationRule.addSlackChannelArn('SlackChannel', props.slackChannelConfigurationArn);
            }
            if (props.notificationTopic) {
                this.pipelineNotificationRule.addTopic(props.notificationTopic);
                props.notificationTopic.grantPublish(underlyingPipeline.role);
            }
            else if (props.notificationTopicArn) {
                const topic = this.pipelineNotificationRule.addTopicArn('NotificationTopic', props.notificationTopicArn);
                topic.grantPublish(underlyingPipeline.role);
            }
        }
    }
    addStage(stage, options) {
        return this.pipeline.addStage(stage, options);
    }
    addWave(id, options) {
        return this.pipeline.addWave(id, options);
    }
}
exports.CdkPipeline = CdkPipeline;
//# sourceMappingURL=cdk-pipeline.js.map